"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.command = void 0;
const apphosting = require("../gcp/apphosting");
const logger_1 = require("../logger");
const command_1 = require("../command");
const projectUtils_1 = require("../projectUtils");
exports.command = new command_1.Command("apphosting:rollouts:create <backendId> <buildId>")
    .description("Create a build for an App Hosting backend")
    .option("-l, --location <location>", "Specify the region of the backend", "us-central1")
    .option("-i, --id <rolloutId>", "Id of the rollout. If not present, autogenerate a random id", "")
    .before(apphosting.ensureApiEnabled)
    .action(async (backendId, buildId, options) => {
    const projectId = (0, projectUtils_1.needProjectId)(options);
    const location = options.location;
    const rolloutId = options.buildId ||
        (await apphosting.getNextRolloutId(projectId, location, backendId));
    const build = `projects/${projectId}/backends/${backendId}/builds/${buildId}`;
    const op = await apphosting.createRollout(projectId, location, backendId, rolloutId, {
        build,
    });
    logger_1.logger.info(`Started a rollout for backend ${backendId} with build ${buildId}.`);
    logger_1.logger.info("Check status by running:");
    logger_1.logger.info(`\tfirebase apphosting:rollouts:list --location ${location}`);
    return op;
});
